import {
  select,
  csv,
  scaleLinear,
  scaleTime,
  extent,
  axisLeft,
  axisBottom,
  scaleOrdinal,
  schemeCategory10,
  format
} from 'd3';
import { colorLegend } from './colorLegend';
import { loadData } from './loadData';
import { scatterplot } from './scatterplot';

const colorScale = scaleOrdinal();
const colorValue = d => d.artist;
var artists;
var selectedArtist;
var data;

const svg = select('svg');
const width = +svg.attr('width');
const height = +svg.attr('height');
const margin = { top: 60, right: 30, bottom: 88, left: 200 };
const innerWidth = width - margin.left - margin.right;
const innerHeight = height - margin.top - margin.bottom;
const title = 'Top 10 Artists, Total Plays per Month';

const scatterG = svg.append('g')
  .attr('transform', `translate(${margin.left},${margin.top})`);
const colorLegendG = svg.append('g')
  .attr('transform', `translate(20, 70)`);

  svg.append('text')
    .attr('class', 'title')
    .attr('x', (width + margin.left) / 2)
    .attr('y', 45)
    .text(title);

loadData().then(processedData => {
  data = processedData;
  artists = [...new Set(data.map(d => d.artist))];
  var artistPlays = {};
  artists.forEach(a => { artistPlays[a] = 0; });
  data.forEach(d => { artistPlays[d.artist] += d.plays; });

  //https://stackoverflow.com/questions/1069666/sorting-javascript-object-by-property-value
  var sortable = [];
  for (var artist in artistPlays) {
      sortable.push([artist, artistPlays[artist]]);
  }
  sortable.sort((a, b) => { return b[1] - a[1]; });
  artists = sortable.map(d => d[0]);
  //console.log(artists);
  //console.log(sortable);
  render();
});

const onClick = d => {
  selectedArtist = d;
  render();
};

const render = () => {
  colorScale
    .domain(artists)
    .range(schemeCategory10); 
  
  colorLegendG.call(colorLegend, {
    colorScale,
    circleRadius: 8,
    spacing: 20,
    textOffset: 12,
    backgroundRectWidth: 135,
    onClick,
    selectedArtist
  });
  
  scatterG.call(scatterplot, {
    data,
    colorScale,
    colorValue,
    selectedArtist,
    innerWidth,
    innerHeight,
    circleRadius: 3
  });
};
 

























/*
const width = +svg.attr('width');
const height = +svg.attr('height');
const artistList = ['Coldplay', 'Death Cab for Cutie', 'Japanese Breakfast', 'The Beatles', 'The Front Bottoms', 'The Hotelier', 'The Strokes', 'Wilco', 'Zox', 'blink-182'];

const xValue = d => d.month;
const xAxisLabel = 'Time';

const yValue = d => d.plays;
const yAxisLabel = 'Plays';

const title = 'Top 10 Artists, Total Plays per Month';
const circleRadius = 4;
const legendOffset = 20;

const margin = { top: 60, right: 140, bottom: 88, left: 80 };
const innerWidth = width - margin.left - margin.right;
const innerHeight = height - margin.top - margin.bottom;

var plays = []; 
var selectedArtist;

const colorScale = scaleOrdinal()
  .domain(artistList)
  .range(schemeCategory10);

const onClick = d => {
  selectedArtist = d;
  console.log(d);
  render(plays);
};

const render = data => {
  const xScale = scaleTime()
    .domain(extent(data, xValue))
    .range([0, innerWidth])
    .nice();
    
  /*
  // Find max play count
  var maxPlays = 0;
  data.forEach(d => {
    var [lMin, lMax] = extent(Object.values(d.artistPlays))
    maxPlays = maxPlays > lMax ? maxPlays : lMax;
  });
  
  
  const yScale = scaleLinear()
    .domain(extent(data, yValue))
    .range([innerHeight, 0])
    .nice();
  
  // Move the legend to the right of the graph
  const colorLegendG = svg.append('g')
    .attr('transform', `translate(${width - margin.right + legendOffset},130)`);
  
  const g = svg.append('g')
    .attr('transform', `translate(${margin.left},${margin.top})`);
  
  const xAxis = axisBottom(xScale)
    .ticks(10)
    .tickSize(-innerHeight)
    .tickPadding(15);
  
  const xAxisG = g.append('g').call(xAxis)
    .attr('transform', `translate(0,${innerHeight})`);
  
  xAxisG.select('.domain').remove();
  
  xAxisG.append('text')
      .attr('class', 'axis-label')
      .attr('y', 50)
      .attr('x', innerWidth / 2)
      .attr('fill', 'black')
      .text(xAxisLabel);

  const yAxisTickFormat = number =>
    format('.2s')(number)
      .replace('.0', '');
  
  const yAxis = axisLeft(yScale)
    .tickSize(-innerWidth)
    .tickPadding(10)
    .tickFormat(yAxisTickFormat);
  
  const yAxisG = g.append('g').call(yAxis);
  yAxisG.selectAll('.domain').remove();
  
  yAxisG.append('text')
      .attr('class', 'axis-label')
      .attr('y', -65)
      .attr('x', -innerHeight / 2)
      .attr('fill', 'black')
      .attr('transform', `rotate(-90)`)
      .attr('text-anchor', 'middle')
      .text(yAxisLabel);
 
/*  
  const areaGenerator = line()
    .x(d => xScale(xValue(d)))
    .y(d => yScale(yValue(d)));
 
  g.append('path')
      .attr('class', 'line-path')
      .attr('d', areaGenerator(data));
*/

  /*
    artistList.forEach(a => {
    var artist = g.selectAll('.' + a).data(data);
    var artistEnter = artist.enter().append('circle')
      .attr('class', a)
      .attr('cy', d => yScale(yValue(d, a)))
      .attr('cx', d => xScale(xValue(d)))
      .attr('r', circleRadius)
      .attr('fill', colorScale(a));
    artist.merge(artistEnter)
      .attr('opacity', 0.4);
    });
  
  var points = g.selectAll('circle').data(data);
  var pointsEnter = points.enter().append('circle');
  points.merge(pointsEnter)
    .attr('class', d => d.artist)
    .attr('cy', d => yScale(yValue(d)))
    .attr('cx', d => xScale(xValue(d)))
    .attr('r', circleRadius)
    .attr('fill', d => colorScale(d.artist))
    .attr('opacity', d => (d.artist === selectedArtist) ? 1 : 0.05);  
      
  svg.append('text')
      .attr('class', 'title')
      .attr('x', width / 2)
      .attr('y', 45)
      .text(title);

  colorLegendG.call(colorLegend, {
    colorScale,
    circleRadius: 5,
    spacing: 20,
    textOffset: 12,
    backgroundRectWidth: 235,
    onClick,
    selectedArtist
  });
};

csv('https://vizhub.com/OxfordComma/datasets/playcounts_bymonth_byartist.csv')
  .then(data => {
   /*
    var newData = {
      months: [],
      artistPlays: {}
    };
    data.forEach(d => {
      newData.months.push(new Date(
        d.year_month.slice(0, 4), 
        d.year_month.slice(4, 6)
      ));
    });
    artistList.forEach(a => {
      newData.artistPlays[a] = data.map(d => d[a]);
    });
    //console.log(newData)
    render(newData);
    ///////////////////////
    data.forEach(d => {
      d.artistPlays = {};
      d.year_month = new Date(
        d.year_month.slice(0, 4), 
        d.year_month.slice(4, 6)
      );
      artistList.forEach(a => {
        d[a] = +d[a];
        d.artistPlays[a] = +d[a];
      });
    });
    plays = data;
    render(plays);
    
    var newData = [];
    data.forEach(d => {
      artistList.forEach(a => {
        newData.push({
          month: new Date(d.year_month.slice(0, 4), 
                          d.year_month.slice(4, 6)), 
          artist: a,
          plays: +d[a]
        });
      });
    });
    plays = newData;
    console.log(newData); 
    render(plays);
  });
*/