import { 
  select, 
  scaleTime, 
  scaleLinear, 
  extent,
  axisLeft,
  axisBottom,
  format
} from 'd3';

export const scatterplot = (selection, props) => {
  const {
    data,
    colorScale,
    colorValue,
    selectedArtist,
    innerWidth,
    innerHeight,
    circleRadius
  } = props;
  
  //console.log(data); 
 
  const xValue = d => d.month;
  const xAxisLabel = 'Time';

  const yValue = d => d.plays;
  const yAxisLabel = 'Plays';
  
  const xScale = scaleTime()
    .domain(extent(data, xValue))
    .range([0, innerWidth])
    .nice();
    
  const yScale = scaleLinear()
    .domain(extent(data, yValue))
    .range([innerHeight, 0])
    .nice();
  
  const xAxis = axisBottom(xScale)
    .ticks(10)
    .tickSize(-innerHeight)
    .tickPadding(15);
  
  const xAxisG = selection.append('g').call(xAxis)
    .attr('transform', `translate(0,${innerHeight})`);
  
  xAxisG.select('.domain').remove();
  
  xAxisG.append('text')
      .attr('class', 'axis-label')
      .attr('y', 50)
      .attr('x', innerWidth / 2)
      .attr('fill', 'black')
      .text(xAxisLabel);

  const yAxisTickFormat = number =>
    format('.2s')(number)
      .replace('.0', '');
  
  const yAxis = axisLeft(yScale)
    .tickSize(-innerWidth)
    .tickPadding(5)
    .tickFormat(yAxisTickFormat);
  
  const yAxisG = selection.append('g').call(yAxis);
  yAxisG.selectAll('.domain').remove();
  
  yAxisG.append('text')
      .attr('class', 'axis-label')
      .attr('y', -35)
      .attr('x', -innerHeight / 2)
      .attr('fill', 'black')
      .attr('transform', `rotate(-90)`)
      .attr('text-anchor', 'middle')
      .text(yAxisLabel);
  
    /*
  const lineGenerator = line()
    .x(d => xScale(xValue(d)))
    .y(d => yScale(yValue(d)));
 
  selection.append('path')
      .attr('class', 'line-path')
      .attr('d', lineGenerator(data))
      .attr('color', 'blue');
  */
  
  const points = selection.selectAll('.artist').data(data);
  const pointsEnter = points
    .enter().append('circle')
      .attr('class', 'artist')
      .attr('cy', d => yScale(yValue(d)))
      .attr('cx', d => xScale(xValue(d)))
      .attr('r', circleRadius)
      .attr('fill', d => colorScale(d.artist));
   points.merge(pointsEnter)
      .attr('opacity', d =>
        (!selectedArtist || d.artist === selectedArtist) ? 1 : 0.05);
};